/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  Copyright (C) 2002, 2003 Tim Waugh <twaugh@redhat.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>

#include <glib.h>
#include <cups/cups.h>
#include <cups/language.h>
#include <locale.h>
#include <time.h>

unsigned int cups_num_printers = 0;
GList *cups_printers = NULL;
gchar *singlejob_printer = NULL;
gchar *singlejob_jobname = NULL;

static http_t *my_connection = NULL;

/* 
 * Get the information from the CUPS server.  If that fails, LPRng is
 * being used.
 */
unsigned int
get_num_jobs (void)
{
	http_t *http;
	ipp_t *request = ippNew(), *answer;
	ipp_attribute_t *attr;
	const char *attributes[] = { "job-id", "job-printer-uri", "job-name" };
	char *lang = setlocale (LC_MESSAGES, NULL);
	int this_num_jobs = 0;
	GList *printers = NULL;

	if (!my_connection) {
		http_encryption_t encryption = cupsEncryption ();
		my_connection = httpConnectEncrypt (cupsServer (), ippPort (),
						    encryption);
	}

	http = my_connection;
	if (!http)
		return 0;

	ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
		      "attributes-charset", NULL, "utf-8");
	ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
		      "attributes-natural-language", NULL, lang);
	ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_NAME,
		      "requesting-user-name", NULL, cupsUser ());
	ippAddStrings (request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
		       "requested-attributes",
		       sizeof (attributes) / sizeof (attributes[0]),
		       NULL, attributes);
	ippAddBoolean (request, IPP_TAG_OPERATION, "my-jobs", 1);
	ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_URI, "job-uri",
                      NULL, "ipp://localhost/jobs/");

	request->request.op.operation_id = IPP_GET_JOBS;
	request->request.op.request_id = 1;
	answer = cupsDoRequest (http, request, "/");
	if (!answer || answer->request.status.status_code > IPP_OK_CONFLICT)
		return 0;

	for (attr = answer->attrs; attr; attr = attr->next) {
		int got_job = 0;
		char *printer = NULL;
		char *job = NULL;

		while (attr && attr->group_tag != IPP_TAG_JOB)
			attr = attr->next;

		if (!attr)
			break;

		for (; attr && attr->group_tag == IPP_TAG_JOB;
		     attr = attr->next) {
			if (!strcmp (attr->name, "job-id") &&
			    attr->value_tag == IPP_TAG_INTEGER) {
				got_job = 1;
			} else if (!strcmp (attr->name, "job-printer-uri") &&
				   attr->value_tag == IPP_TAG_URI) {
				char *p = attr->values[0].string.text;
				char *q = strrchr (p, '/');
				if (q) {
					if (*++q)
						printer = q;
				}
			} else if (!strcmp (attr->name, "job-name") &&
				   attr->value_tag == IPP_TAG_NAME)
				job = attr->values[0].string.text;
		}

		if (got_job && printer) {
			GList *l;

			this_num_jobs++;
			if (this_num_jobs == 1) {
				if (singlejob_printer)
					g_free (singlejob_printer);
				if (singlejob_jobname)
					g_free (singlejob_jobname);

				singlejob_printer = g_strdup_printf ("%s",
								     printer);
				singlejob_jobname = g_strdup_printf ("%s",
								     job);
			}

			for (l = printers; l != NULL; l = l->next)
				if (!strcmp ((char *) l->data, printer))
					break;

			if (!l)
				printers = g_list_prepend (printers,
							   g_strdup (printer));
		}

		if (!attr)
			break;
	}

	ippDelete (answer);
	
	{
		GList *orig_l = cups_printers, *l = orig_l;
		cups_printers = printers;
		while (l) {
			g_free ((char *) l->data);
			l = l->next;
		}
		g_list_free (orig_l);
	}

	return this_num_jobs;
}
