/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2002 CodeFactory AB
 * Copyright (C) 2002 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2002 Tim Waugh <twaugh@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <signal.h>
#include <string.h>
#include <sys/wait.h>
#include <gtk/gtk.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include <libgnomeui/libgnomeui.h>
#include "lp-intl.h"
#include "lpegg.h"

gboolean
session_die (GnomeClient *client, gpointer client_data)
{
	gtk_main_quit ();

	return TRUE;
}

gboolean
session_save (GnomeClient *client, gpointer client_data)
{
	return TRUE;
}

static void
reap (int sig)
{
	wait (NULL);
}

int
main (int argc, char *argv[])
{
	gint          i;
	LpEgg     *lpegg;
	GnomeClient  *client;
	struct sigaction act;

	bindtextdomain (GETTEXT_PACKAGE, EGGCUPS_LOCALEDIR);  
        bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	act.sa_handler = reap;
	act.sa_flags = SA_NOCLDSTOP;
	sigaction (SIGCHLD, &act, NULL);

	i = 1;
	while (i < argc) {
		const gchar *arg = argv[i];
      
		if (strcmp (arg, "-?") == 0) {
			g_printerr ("Usage: %s [--debug]\n", argv[0]);
			return 0;
		}
      
		++i;
	}

	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, 
			    argc, argv, NULL);

	client = gnome_master_client ();
	
	g_signal_connect (client,
			  "save_yourself",
			  G_CALLBACK (session_save),
			  NULL);

	g_signal_connect (client,
			  "die",
			  G_CALLBACK (session_die),
			  NULL);

	lpegg = lpegg_new ();

	gtk_main ();

	return 0;
}
